/* PFFT headers include FFTW-MPI, and FFTW-MPI headers include FFTW and MPI.
   No matter what, we only need to include one file.

   This header is internal.  We do not bother to have a serial and parallel version.
*/

#include "vdw_radial.h"


#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_PFFT
#include <pfft.h>
#else
#ifdef HAVE_MPI
#include <fftw3-mpi.h>
#else
#include <fftw3.h>
#endif
#endif

#define VDW_PI 3.14159265358979323846

struct vdwxc_unit_cell
{
    double vec[9];
    int Nglobal[3];
    int Nlocal[3];
    int offset[3];
    double dV;
};

struct vdwxc_data_obj
{
    int initialized;
    int functional;
    double Z_ab;
    double q_cut;
    struct vdwxc_unit_cell cell;
    struct vdwxc_unit_cell icell;

    int fft_type;
#ifdef HAVE_MPI
    MPI_Comm mpi_comm;
#endif

    int mpi_rank;
    int mpi_size;

    int Ng; // Number of local real space points
    int Nglobal; // Number of global real space points
    int gLDA;

    int pfft_grid[2];

    double* q0_g;
    double complex* work_ka; // theta_ga (padded), theta_ka, and F_ka.

    //int Nk; // Number of local reciprocal space points
    int kLDA; // Shortest dimension of k stored in reduced r2c-format.

    double* dq0_drho_g;
    double* dq0_dsigma_g;

    fftw_plan plan_r2c;
    fftw_plan plan_c2r;
    //fftw_plan radial_plan_r2c;
    //fftw_plan radial_plan_c2r;

#ifdef HAVE_PFFT
    pfft_plan pfft_plan_r2c;
    pfft_plan pfft_plan_c2r;
    MPI_Comm mpi_cart_comm;
#endif
    int errorcode;
};


void vdwxc_calculate_thetas(int nx, int ny, int nz, int leading_dim,
                            double* rho, double* q0, double* output);

void vdwxc_lda_correlation(int Ng, double* rho_g, double* eps_g, double* dedn_g);


#ifdef HAVE_MPI
void vdwxc_set_communicator(vdwxc_data data, MPI_Comm mpi_comm);
//void vdw_set_communicator_pfft(vdwxc_data data, MPI_Comm mpi_comm,
//                               int nproc1, int nproc2)
#endif
